==> Eri-mfvp-deployment-toolkit-1.5.07.001.el7.x86_64.tgz: opt/mfvp/deploypattern/lib/python2.7/site-packages/ericsson/deploypattern/lib/inventory.py <==
"""
This software has been developed by Ericsson.

Copyright (c) 2016 Ericsson, Inc.

COPYRIGHT:
    This file is the property of Ericsson.
    It cannot be copied, used, or modified without obtaining
    an authorization from the authors or a mandated
    member of Ericsson.
    If such an authorization is provided, any modified version or
    copy of the software must contain this header.

 WARRANTIES:
    This software is made available by the authors in the hope
    that it will be useful, but without any warranty.
    Ericsson.com is not liable for any consequence related to the
    use of the provided software.
"""

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import sys
import logging

from ansible.parsing.dataloader import DataLoader
from ansible.vars import VariableManager
from ansible.inventory import Inventory
from ansible.inventory.group import Group
from ansible.inventory.host import Host

from ericsson.deploypattern.lib.config import Config
from ericsson.deploypattern.lib.config import get_ssh_key_file, get_remote_user
from ericsson.deploypattern.lib.utils import save_yaml, load_yaml

import ericsson.deploypattern.lib.errors as errors

logger = logging.getLogger(__name__)

SAVE_INVENTORY_FILE = "cluster_inventory.yaml"


def inventory_command(cmd):
    if cmd.list:
        list_inventory(cmd.format)
    if cmd._import:
        save_inventory(cmd._import)
    if cmd.export:
        export_inventory(cmd.export)


def build_inventory(inventory=None, node_list=None):
    """
    Build or Update Inventory object from node list with group
    nodes:
       - group1:
           - name: host1
             ipaddress: addr1
           - name: host2
             ipaddress: addr2
       - group2:
           - name: host3
             ipaddress: addr3
       - group3:
           - children: group1
           - children: group2
       - name: host4
         ipaddress: host4
       - name: host5
         ipaddress: addr5

    :param node_list:
    :return: Inventory
    """

    if inventory is None:
        inventory = Inventory(loader=DataLoader(),
                              variable_manager=VariableManager(),
                              host_list=None)

    all_group = inventory.get_group('all')
    host_list = []
    for n in node_list:
        if 'name' in n and 'ipaddress' in n:
            name, address = get_ip_and_name(n)
            new_host = create_new_host(address, name)
            host_list.append(new_host)
        # We are in a group
        else:
            for group, nodegrouplist in n.iteritems():
                new_group = create_new_group(group, nodegrouplist,
                                             inventory)
                all_group.add_child_group(new_group)
                inventory.add_group(new_group)
                logger.debug("New Group: %s=%s" % (
                    new_group.name, new_group.get_hosts()))

    ungrouped = inventory.get_group('ungrouped')
    for host in host_list:
        all_group.add_host(host)
        ungrouped.add_host(host)

    logger.debug("Group: %s hosts = %s" % (
        all_group.name, all_group.get_hosts()))
    logger.debug("Group: %s child = %s" % (
        all_group.name, all_group.child_groups))
    return inventory


def get_ip_and_name(host):
    _name = None
    _address = None
    try:
        for key, value in host.iteritems():
            if key == 'name':
                if not isinstance(value, str):
                    raise errors.PatternError(
                        "bad structure in pattern: "
                        "bad value for key \'name\' in group %s" % group)
                _name = value
            elif key == 'ipaddress':
                if not isinstance(value, str):
                    raise errors.PatternError(
                        "bad structure in pattern: "
                        "bad value for key \'ipaddress\' in group %s" % group)
                _address = value
    except Exception as e:
        raise errors.PatternError("bad structure in pattern: %s" % e.message)
    return _name, _address


def create_new_host(address, name):
    logger.debug(":: Creating Host: %s:%s" % (name, address))
    new_host = Host(name=name)
    new_host.address = address
    new_host.set_variable('ansible_host', address)

    private_key_file =get_ssh_key_file()
    if private_key_file:
        new_host.set_variable('ansible_ssh_private_key_file',
                              private_key_file)
    remote_user = get_remote_user()
    if remote_user:
        new_host.set_variable('ansible_user',
                              remote_user)
    return new_host


def create_new_group(groupname, grouplist, inventory):
    logger.debug(":: Creating Group: %s" % groupname)
    new_group = Group(groupname)
    product_group = None

    if grouplist is not None:
        for g in grouplist:

            if 'product' in g:
                product = g['product']
                if not isinstance(product, str):
                    raise errors.PatternError(
                        "bad structure in pattern: "
                        "bad value for key \'product\' in group %s" % groupname)
                logger.debug(" > add product Group: %s" % product)
                product_group = inventory.get_group(product)
                if product_group is None:
                    logger.debug(":: Creating Product Group: %s" % product)
                    product_group = Group(product)
                    all_group = inventory.get_group('all')
                    all_group.add_child_group(product_group)
                    inventory.add_group(product_group)
                    new_group.add_child_group(product_group)

            if 'children' in g:
                child = g['children']
                if not isinstance(child, str):
                    raise errors.PatternError(
                        "bad structure in pattern: "
                        "bad value for key \'children\' in group %s" % groupname)
                logger.debug(" > add children Group: %s" % child)
                new_group.add_child_group(inventory.get_group(child))

            else:
                name, address = get_ip_and_name(g)
                if name is None and address is None:
                    raise errors.PatternError(
                        "bad structure in pattern:"
                        "key %s unknown in group %s" % (name, address))
                new_host = create_new_host(address, name)
                if product_group:
                    product_group.add_host(new_host)
                else:
                    new_group.add_host(new_host)

    return new_group


def list_inventory(format=None):
    if format == 'ansible':
        list_inventory_ansible_format()
    elif format == 'yaml':
        list_inventory_yaml_format()
    else:
        sys.stderr.write("\n >>> Unknown format: %s" % format)


def list_inventory_ansible_format():
    node_list = get_saved_node_list()
    if node_list is not None:
        inventory_obj = build_inventory(node_list=node_list)
        export_inventory_obj(inventory_obj, sys.stdout)
        sys.stderr.write("\n")
    else:
        sys.stderr.write("\n >>> No inventory saved found !\n\n")


def list_inventory_yaml_format():
    node_list_file = get_saved_inventory_path()
    if os.path.exists(node_list_file):
        with open(node_list_file) as fd:
            data = fd.read()
            sys.stdout.write(data)
    else:
        sys.stderr.write("\n >>> No inventory saved found !\n\n")


def export_inventory(_file):
    node_list = get_saved_node_list()
    if node_list is not None:
        inventory_obj = build_inventory(node_list=node_list)
        with open(_file, 'w+') as fd:
            export_inventory_obj(inventory_obj, fd)
        fd.close()
    else:
        sys.stderr.write("\n >>> No inventory saved found !\n\n")


def save_inventory(_file):
    node_list = load_yaml(_file)
    save_yaml_inventory(node_list)


def get_saved_node_list():
    data = None
    inventory_path = get_saved_inventory_path()
    if os.path.exists(inventory_path):
        data = load_yaml(inventory_path)
    return data


def get_saved_inventory_path():
    path = Config().get('global', 'path')
    return os.path.join(path , 'inventory', SAVE_INVENTORY_FILE)


def export_inventory_obj(inventory, fd):
    ungrouped = inventory.get_group('ungrouped')
    for host in ungrouped.hosts:
        fd.write("%s ansible_host=%s\n" % (host.name, host.address))

    for grp in inventory.get_group_dict():
        if grp != 'all' and grp != 'ungrouped':
            group = inventory.get_group(grp)

            fd.write("\n[%s]\n" % group)
            if group.hosts:
                for host in group.hosts:
                    #print("HOST= %s" % host.__dict__)
                    fd.write("%s ansible_host=%s\n" % (host.name, host.address))

            if group.child_groups:
                fd.write("\n[%s:children]\n" % group)
                for child in group.child_groups:
                    fd.write("%s\n" % child)

            if group.vars:
                fd.write("\n[%s:vars]\n" % group)
                for var in group.vars:
                    for k, v in var:
                        fd.write("%s=%s\n" % (k, v))


def save_yaml_inventory(node_list, _file=None):
    if _file is None:
        _file = SAVE_INVENTORY_FILE

    path = Config().get('global', 'path')
    inventory_path = os.path.join(path , 'inventory')
    if not os.path.exists(inventory_path):
        os.makedirs(inventory_path)

    full_path = os.path.join(inventory_path, _file)
    logger.debug("Saving inventory in %s" % full_path)
    save_yaml(full_path, node_list)

==> Eri-mfvp-deployment-toolkit-2.0.07.017.el7.x86_64.tgz: opt/mfvp/deploypattern/lib/python2.7/site-packages/ericsson/deploypattern/lib/inventory.py <==
"""
This software has been developed by Ericsson.

Copyright (c) 2016 Ericsson, Inc.

COPYRIGHT:
    This file is the property of Ericsson.
    It cannot be copied, used, or modified without obtaining
    an authorization from the authors or a mandated
    member of Ericsson.
    If such an authorization is provided, any modified version or
    copy of the software must contain this header.

 WARRANTIES:
    This software is made available by the authors in the hope
    that it will be useful, but without any warranty.
    Ericsson.com is not liable for any consequence related to the
    use of the provided software.
"""

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import sys
import logging

from ansible.parsing.dataloader import DataLoader
from ansible.inventory.manager import InventoryManager
from ansible.inventory.host import Host

from ericsson.deploypattern.lib.config import Config
from ericsson.deploypattern.lib.config import get_ssh_key_file, get_remote_user
from ericsson.deploypattern.lib.utils import save_yaml, load_yaml

import ericsson.deploypattern.lib.errors as errors

logger = logging.getLogger(__name__)

SAVE_INVENTORY_FILE = "cluster_inventory.yaml"


def inventory_command(cmd):
    if cmd.list:
        list_inventory(cmd.format)
    if cmd._import:
        save_inventory(cmd._import)
    if cmd.export:
        export_inventory(cmd.export)


def build_inventory(inventory=None, node_list=None):
    """
    Build or Update Inventory object from node list with group
    nodes:
       - group1:
           - name: host1
             ipaddress: addr1
           - name: host2
             ipaddress: addr2
       - group2:
           - name: host3
             ipaddress: addr3
       - group3:
           - children: group1
           - children: group2
       - name: host4
         ipaddress: host4
       - name: host5
         ipaddress: addr5

    Correction with Ansible >= 2.5
    :param node_list:
    :return: Inventory
    """

    if inventory is None:
        inventory = InventoryManager(loader=DataLoader(),sources=None)

    host_list = []
    for n in node_list:
        # Case node outside any group
        # Create Host add it to host list
        if 'name' in n and 'ipaddress' in n:
            name, address = get_ip_and_name(n)
            new_host = create_new_host(address, name)
            host_list.append(new_host)
        # Case group
        # with eventually children group
        else:
            for group, nodegrouplist in n.iteritems():
                # With Ansible >= 2.5 add group with group name, no more
                # group object
                inventory.add_group(group)
                inventory._inventory.add_child('all',group)
                # Add Host or child to this group
                for g in nodegrouplist:
                    # Case sub group
                    if 'children' in g:
                        child = g['children']
                        if not isinstance(child, str):
                            raise errors.PatternError(
                                    "bad structure in pattern: "
                                    "bad value for key \'children\' in group %s" \
                                    % group)
                        inventory._inventory.add_child(group, child)
                    # Case node to add to group
                    # Create Host, add to inventory and as child to group
                    else:
                        name,address = get_ip_and_name(g)
                        host = create_new_host(address, name)
                        inventory._inventory.add_host(name)
                        inventory._inventory.add_child(group, name)

    # Add nodes without group, add them to 2 groups
    # all and also ungrouped (default) groups
    for host in host_list:
        inventory._inventory.add_host(host,'all')
        inventory._inventory.add_host(host,'ungrouped')

    return inventory


def get_ip_and_name(host):
    _name = None
    _address = None
    try:
        for key, value in host.iteritems():
            if key == 'name':
                if not isinstance(value, str):
                    raise errors.PatternError(
                        "bad structure in pattern: "
                        "bad value for key \'name\' in group %s" % group)
                _name = value
            elif key == 'ipaddress':
                if not isinstance(value, str):
                    raise errors.PatternError(
                        "bad structure in pattern: "
                        "bad value for key \'ipaddress\' in group %s" % group)
                _address = value
    except Exception as e:
        raise errors.PatternError("bad structure in pattern: %s" % e.message)
    return _name, _address


def create_new_host(address, name):
    logger.debug(":: Creating Host: %s:%s" % (name, address))
    new_host = Host(name=name)
    new_host.address = address
    new_host.set_variable('ansible_host', address)

    private_key_file =get_ssh_key_file()
    if private_key_file:
        new_host.set_variable('ansible_ssh_private_key_file',
                              private_key_file)
    remote_user = get_remote_user()
    if remote_user:
        new_host.set_variable('ansible_user',
                              remote_user)
    return new_host




def list_inventory(format=None):
    if format == 'ansible':
        list_inventory_ansible_format()
    elif format == 'yaml':
        list_inventory_yaml_format()
    else:
        sys.stderr.write("\n >>> Unknown format: %s" % format)


def list_inventory_ansible_format():
    node_list = get_saved_node_list()
    if node_list is not None:
        inventory_obj = build_inventory(node_list=node_list)
        export_inventory_obj(inventory_obj, sys.stdout)
        sys.stderr.write("\n")
    else:
        sys.stderr.write("\n >>> No inventory saved found !\n\n")


def list_inventory_yaml_format():
    node_list_file = get_saved_inventory_path()
    if os.path.exists(node_list_file):
        with open(node_list_file) as fd:
            data = fd.read()
            sys.stdout.write(data)
    else:
        sys.stderr.write("\n >>> No inventory saved found !\n\n")


def export_inventory(_file):
    node_list = get_saved_node_list()
    if node_list is not None:
        inventory_obj = build_inventory(node_list=node_list)
        with open(_file, 'w+') as fd:
            export_inventory_obj(inventory_obj, fd)
        fd.close()
    else:
        sys.stderr.write("\n >>> No inventory saved found !\n\n")


def save_inventory(_file):
    node_list = load_yaml(_file)
    save_yaml_inventory(node_list)


def get_saved_node_list():
    data = None
    inventory_path = get_saved_inventory_path()
    if os.path.exists(inventory_path):
        data = load_yaml(inventory_path)
    return data


def get_saved_inventory_path():
    path = Config().get('global', 'path')
    return os.path.join(path , 'inventory', SAVE_INVENTORY_FILE)


def export_inventory_obj(inventory, fd):
    ungrouped = inventory.get_group('ungrouped')
    for host in ungrouped.hosts:
        fd.write("%s ansible_host=%s\n" % (host.name, host.address))

    for grp in inventory.get_group_dict():
        if grp != 'all' and grp != 'ungrouped':
            group = inventory.get_group(grp)

            fd.write("\n[%s]\n" % group)
            if group.hosts:
                for host in group.hosts:
                    #print("HOST= %s" % host.__dict__)
                    fd.write("%s ansible_host=%s\n" % (host.name, host.address))

            if group.child_groups:
                fd.write("\n[%s:children]\n" % group)
                for child in group.child_groups:
                    fd.write("%s\n" % child)

            if group.vars:
                fd.write("\n[%s:vars]\n" % group)
                for var in group.vars:
                    for k, v in var:
                        fd.write("%s=%s\n" % (k, v))


def save_yaml_inventory(node_list, _file=None):
    if _file is None:
        _file = SAVE_INVENTORY_FILE

    path = Config().get('global', 'path')
    inventory_path = os.path.join(path , 'inventory')
    if not os.path.exists(inventory_path):
        os.makedirs(inventory_path)

    full_path = os.path.join(inventory_path, _file)
    logger.debug("Saving inventory in %s" % full_path)
    save_yaml(full_path, node_list)
